"""A Python package for scraping Japanese stock information from various web sites."""

from __future__ import annotations

from .jpx import Jpx
from .kabuyoho import Kabuyoho

__version__ = "0.0.5"

kabuyoho = Kabuyoho()
jpx = Jpx()



"""Scraper for jpx.co.jp"""
from __future__ import annotations

import functools
import urllib.parse

import pandas as pd
import requests
from bs4 import BeautifulSoup

from ..base import Website
from ..exceptions import KabupyError


class Jpx(Website):
    """An object for jpx.co.jp"""

    def __init__(self) -> None:
        self.url = "https://www.jpx.co.jp"

    @functools.cached_property
    def issues_link(self) -> str:
        """Return a link to the issues list."""
        response = requests.get("https://www.jpx.co.jp/markets/statistics-equities/misc/01.html", timeout=10)
        response.raise_for_status()
        response.encoding = response.apparent_encoding
        soup = BeautifulSoup(response.text, "html.parser")
        href = soup.select_one('th:-soup-contains("東証上場銘柄一覧") + td>a')
        if href is None:
            raise KabupyError("no link was found.")
        href = href.attrs["href"]
        return urllib.parse.urljoin(self.url, href)

    @functools.cached_property
    def issues(self):
        """Return a list of issues."""
        link = self.issues_link
        response = requests.get(link, timeout=10)
        response.raise_for_status()
        return pd.read_excel(
            response.content,
            names=[
                "date",
                "security_code",
                "name",
                "category",
                "33_industry_code",
                "33_industry_category",
                "17_industry_code",
                "17_industry_category",
                "market_capitalization_code",
                "market_capitalization_category",
            ],
        ).to_dict("records")



"""Base class for webpage"""
from __future__ import annotations

import re
from datetime import datetime

from money import Money

from ..base import Webpage, webpage_property
from ..util import str2money


class KabuyohoWebpage(Webpage):
    """Base class for kabuyoho webpage"""

    security_code: str

    def term2description(self, term: str) -> str:
        """Get dd text from dt text"""
        res = self.select_one(f'main dt:-soup-contains("{term}") + dd')
        return re.sub(r"\s+", "", res.text)

    @webpage_property
    def price(self) -> Money | None:
        """Price of the stock: 価格"""
        amount = self.soup.select_one('main li p:-soup-contains("株価","(","/",")") + p')
        if amount is None:
            return None
        return str2money(amount.text)

    @webpage_property
    def name(self) -> str | None:
        """Name of the stock: 銘柄名"""
        res = self.soup.select_one(f"main ul:-soup-contains('{self.security_code}') > li")
        if res is None:
            return None
        return res.text

    @webpage_property
    def earnings_release_date(self) -> datetime | None:
        """Earnings release date: 決算発表日"""
        res = self.soup.select_one(f"main ul:-soup-contains('{self.security_code}') > li:last-of-type")
        if res is None:
            return None
        match = re.search(r"(\d{4})/(\d{2})/(\d{2})", res.text)
        if match:
            year, month, day = match.groups()
            return datetime(int(year), int(month), int(day))
        return None



"""Scraper for https://kabuyoho.jp/sp/reportTop"""
from __future__ import annotations

import logging
import re
import urllib.parse

from money import Money

from ..base import Website, webpage_property
from ..util import str2float, str2int, str2money
from .kabuyoho_webpage import KabuyohoWebpage

logger = logging.getLogger(__name__)


class ReportTarget(KabuyohoWebpage):
    """Report target page object."""

    def __init__(self, website: Website, security_code: str | int) -> None:
        self.website = website
        self.security_code = str(security_code)
        self.url = urllib.parse.urljoin(self.website.url, f"sp/reportTarget?bcode={self.security_code}")
        super().__init__()

    # Properties in "price target(目標株価)"

    @webpage_property
    def price_level_to_target(self) -> str | None:
        """Current price level to target price: 目標株価に対する現在の価格が割高か割安か."""
        return None if self.term2description("目標株価から見た株価") == "--" else self.term2description("目標株価から見た株価")

    @webpage_property
    def price_target(self) -> Money | None:
        """Price target: 目標株価(アナリストが発表した目標株価の平均値)"""
        amount = self.select_one('thead:has(>tr>th:-soup-contains("平均")) ~ tbody>tr>td:nth-of-type(1)')
        return str2money(amount.text)

    @webpage_property
    def price_target_ratio_to_previous_week(self) -> float | None:
        """Price target ratio to previous week in %: 目標株価の対前週変化率"""
        amount = self.select_one('thead:has(>tr>th:-soup-contains("平均")) ~ tbody>tr>td:nth-of-type(2)')
        return str2float(amount.text)

    @webpage_property
    def price_target_ratio_to_current_price(self) -> float | None:
        """(price target) / (current price) in %: 目標株価と現在の株価の乖離率"""
        amount = self.select_one('thead:has(>tr>th:-soup-contains("平均")) ~ tbody>tr>td:nth-of-type(3)')
        return str2float(amount.text)

    # Properties in "rating(レーティング)"

    @webpage_property
    def average_analyst_rating(self) -> float | None:
        """Average analyst rating: レーティング(平均)"""
        amount = self.select_one('main section:has(h1:-soup-contains("レーティング")) th:-soup-contains("平均") + td')
        return str2float(amount.text)

    @webpage_property
    def analyst_count(self) -> int | None:
        """Average count: レーティング(人数)"""
        amount = self.select_one('main section:has(h1:-soup-contains("レーティング")) th:-soup-contains("人数") + td')
        amount = re.sub(r"\D", "", amount.text)
        if amount == "":
            amount = "0"
        return int(amount, 10)

    @webpage_property
    def analyst_rating_composition(self) -> dict[str, int]:
        """Analyst rating composition: レーティング(点数の構成)

        Returns:
            dict[str, int]: key: rating("1", "2", "3", "4", and "5"),
                            which respectively means
                            "strong sell(弱気)", "sell(やや弱気)", "hold(中立)", "buy(やや強気)", and "strong buy(強気)"
                            value: the number of analysts
        """
        ratings = ["1", "2", "3", "4", "5"]
        composition = {}
        for rating in ratings:
            res = self.select_one(
                'main h1:-soup-contains("レーティング") + div ' f'tbody tr>th:-soup-contains("({rating}点)") + td'
            )
            composition[rating] = str2int(res.text)
        return composition

    # Properties in "stock index(株価指標)"

    @webpage_property
    def bps(self) -> Money | None:
        """Book-value per share: BPS(実績)"""
        amount = self.select_one('main h2:-soup-contains("株価指標") + table th:-soup-contains("BPS(実績)") + td')
        return str2money(amount.text)

    @webpage_property
    def forward_eps(self) -> Money | None:
        """Forward earnings per share: EPS(予想)"""
        amount = self.select_one('main h2:-soup-contains("株価指標")+table th:-soup-contains("EPS(予想)") + td')
        return str2money(amount.text)

    @webpage_property
    def forward_eps_by_analysts(self) -> Money | None:
        """Forward earnings per share in twelve months based on analysts estimates: EPS(アナリスト12ヶ月後予想)"""
        amount = self.select_one('main h2:-soup-contains("株価指標")+table th:-soup-contains("EPS ※") + td')
        return str2money(amount.text)

    @webpage_property
    def pbr(self) -> float | None:
        """Price to book ratio: PBR"""
        amount = self.select_one('main h2:-soup-contains("株価指標")+table th:-soup-contains("PBR") + td')
        return str2float(amount.text)

    @webpage_property
    def forward_per(self) -> float | None:
        """Forward price to earnings ratio based on company estimates: PER(会予)"""
        amount = self.select_one('main h2:-soup-contains("株価指標")+table th:-soup-contains("PER(会予)") + td')
        return str2float(amount.text)

    @webpage_property
    def forward_per_by_analysts(self) -> float | None:
        """Forward PER in twelve months based on analysts estimates: PER(アナリスト12ヶ月後予想)"""
        amount = self.select_one('main h2:-soup-contains("株価指標")+table th:-soup-contains("PER ※") + td')
        return str2float(amount.text)

    # Properties in "target price range(想定株価レンジ)"

    @webpage_property
    def pbr_based_fair_value(self) -> Money | None:
        """PBR based fair value: 理論株価(PBR基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr>th:-soup-contains("理論株価(PBR基準)") + '
            "td>span:nth-of-type(1)"
        )
        return str2money(value.text)

    @webpage_property
    def pbr_fair(self) -> float | None:
        """PBR when the stock price is at pbr_based_fair_value: 理論株価(PBR基準)の時のPBR"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr>th:-soup-contains("理論株価(PBR基準)") + '
            "td>span:nth-of-type(2)"
        )
        return str2float(value.text)

    @webpage_property
    def pbr_based_ceiling(self) -> Money | None:
        """PBR based ceiling price of the stock: 上値目途(PBR基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PBR基準)")) ~ '
            'tr>th:-soup-contains("上値目途") + td>span:nth-of-type(1)'
        )
        return str2money(value.text)

    @webpage_property
    def pbr_ceiling(self) -> float | None:
        """PBR when the stock price is at pbr_based_ceiling: 下値目途(PBR基準)の時のPBR"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PBR基準)")) ~ '
            'tr>th:-soup-contains("上値目途") + td>span:nth-of-type(2)'
        )
        return str2float(value.text)

    @webpage_property
    def pbr_based_floor(self) -> Money | None:
        """PBR based floor price of the stock: 下値目途(PBR基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PBR基準)")) ~ '
            'tr>th:-soup-contains("下値目途") + td>span:nth-of-type(1)'
        )
        return str2money(value.text)

    @webpage_property
    def pbr_floor(self) -> float | None:
        """PBR when the stock price is at pbr_based_floor: 下値目途(PBR基準)の時のPBR"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PBR基準)")) ~ '
            'tr>th:-soup-contains("下値目途") + td>span:nth-of-type(2)'
        )
        return str2float(value.text)

    @webpage_property
    def per_based_fair_value(self) -> Money | None:
        """PER based fair value: 理論株価(PER基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr>th:-soup-contains("理論株価(PER基準)") + '
            "td>span:nth-of-type(1)"
        )
        return str2money(value.text)

    @webpage_property
    def per_fair(self) -> float | None:
        """PER when the stock price is at per_based_fair_value: 理論株価(PER基準)の時のPER"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr>th:-soup-contains("理論株価(PER基準)") + '
            "td>span:nth-of-type(2)"
        )
        return str2float(value.text)

    @webpage_property
    def per_based_ceiling(self) -> Money | None:
        """PER based ceiling price of the stock: 上値目途(PER基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PER基準)")) ~ '
            'tr>th:-soup-contains("上値目途") + td>span:nth-of-type(1)'
        )
        return str2money(value.text)

    @webpage_property
    def ceiling_per(self) -> float | None:
        """PER when the stock price is at per_based_ceiling: 下値目途(PER基準)の時のPER"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PER基準)")) ~ '
            'tr>th:-soup-contains("上値目途") + td>span:nth-of-type(2)'
        )
        return str2float(value.text)

    @webpage_property
    def per_based_floor(self) -> Money | None:
        """PER based floor price of the stock: 下値目途(PER基準)"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PER基準)")) ~ '
            'tr>th:-soup-contains("下値目途") + td>span:nth-of-type(1)'
        )
        return str2money(value.text)

    @webpage_property
    def per_floor(self) -> float | None:
        """PER when the stock price is at per_based_floor: 下値目途(PER基準)の時のPER"""
        value = self.select_one(
            'main h2:-soup-contains("想定株価レンジ") + '
            'table tr:has(>th:-soup-contains("理論株価(PER基準)")) ~ '
            'tr>th:-soup-contains("下値目途") + td>span:nth-of-type(2)'
        )
        return str2float(value.text)



"""Scraper for kabuyoho.jp"""
from __future__ import annotations

import functools
import logging

from ..base import Website
from .report_dps import ReportDps
from .report_target import ReportTarget
from .report_top import ReportTop

logger = logging.getLogger(__name__)


class Kabuyoho(Website):
    """An object for kabuyoho.jp"""

    def __init__(self) -> None:
        self.url = "https://kabuyoho.jp"

    def stock(self, security_code: str | int) -> Stock:
        """Return Stock object"""
        return Stock(self, security_code)


class Stock:
    """Stock object for kabuyoho.jp"""

    def __init__(self, website: Kabuyoho, security_code: str | int) -> None:
        self.security_code = str(security_code)
        self.website = website

    @functools.cached_property
    def report_top(self) -> ReportTop:
        """Report top page object"""
        return ReportTop(self.website, self.security_code)

    @functools.cached_property
    def report_target(self) -> ReportTarget:
        """Report target page object"""
        return ReportTarget(self.website, self.security_code)

    @functools.cached_property
    def report_dps(self) -> ReportDps:
        """Report DPS page object"""
        return ReportDps(self.website, self.security_code)



"""Base class for webpage"""
from __future__ import annotations

from abc import ABC

import requests
from bs4 import BeautifulSoup
from bs4.element import Tag

from ..errors import ElementNotFoundError


class Webpage(ABC):
    """Base class for website"""

    url: str
    html: str
    soup: BeautifulSoup

    def __init__(self):
        self.load()

    def load(self):
        """Load webpage and set html and soup"""
        response = requests.get(self.url, timeout=10)
        response.raise_for_status()
        self.html = response.text
        self.soup = BeautifulSoup(self.html, "html.parser")

    def select_one(self, selector: str) -> Tag:
        """Select one element from soup"""
        res = self.soup.select_one(selector)
        if not res:
            raise ElementNotFoundError(f"{selector} not found in {self.url}")
        return res

    def select(self, selector: str) -> list[Tag]:
        """Select elements from soup"""
        res = self.soup.select(selector)
        if len(res) == 0:
            raise ElementNotFoundError(f"{selector} not found in {self.url}")
        return res



"""Scraper for https://kabuyoho.jp/sp/reportTop"""
from __future__ import annotations

import logging
import re
import urllib.parse

from money import Money

from ..base import Website, webpage_property
from ..util import str2float, str2money
from .kabuyoho_webpage import KabuyohoWebpage

logger = logging.getLogger(__name__)


class ReportTop(KabuyohoWebpage):
    """Report target page object."""

    def __init__(self, website: Website, security_code: str | int) -> None:
        self.website = website
        self.security_code = str(security_code)
        self.url = urllib.parse.urljoin(self.website.url, f"sp/reportTop?bcode={self.security_code}")
        super().__init__()

    @webpage_property
    def expected_per(self) -> float | None:
        """Expected PER: PER(予)."""
        amount = self.term2description("PER(予)")
        return str2float(amount)

    @webpage_property
    def actual_pbr(self) -> float | None:
        """Actual PBR: PBR(実)."""
        amount = self.term2description("PBR(実)")
        return str2float(amount)

    @webpage_property
    def expected_dividend_yield(self) -> float | None:
        """Expected dividend yield: 配当利回り(予)."""
        amount = self.term2description("配当利回り(予)")
        return str2float(amount)

    @webpage_property
    def market_capitalization(self) -> Money | None:
        """Market Capitalization: 時価総額."""
        amount = self.term2description("時価総額")
        return str2money(amount)

    @webpage_property
    def actual_roa(self) -> float | None:
        """Actual ROA: ROA(実)."""
        amount = self.term2description("ROA(実)")
        return str2float(amount)

    @webpage_property
    def actual_roe(self) -> float | None:
        """Actual ROE: ROE(実)."""
        amount = self.term2description("ROE(実)")
        return str2float(amount)

    @webpage_property
    def equity_ratio(self) -> float | None:
        """Equity ratio: 自己資本率."""
        amount = self.term2description("自己資本比率")
        return str2float(amount)

    @webpage_property
    def signal(self) -> str | None:
        """Signal: シグナル."""
        res = self.term2description("シグナル")
        return re.sub(r"\s+", "", res)

    @webpage_property
    def expected_ordinary_profit(self) -> Money | None:
        """Market Capitalization: 予想経常利益(予)."""
        amount = self.select_one('main dt:-soup-contains("予想経常利益(予)") + dd>p')
        return str2money(amount.text.split("円")[0])

    @webpage_property
    def consensus_expected_ordinary_profit(self) -> Money | None:
        """Market Capitalization: 予想経常利益(コ)."""
        amount = self.select_one('main dt:-soup-contains("予想経常利益(コ)") + dd>p')
        return str2money(amount.text.split("円")[0])

    @webpage_property
    def target_price(self) -> Money | None:
        """Target Price: 目標株価."""
        amount = self.select_one('main dt:-soup-contains("目標株価(コ)") + dd>p>span')
        return str2money(amount.text)

    @webpage_property
    def business_category(self) -> str | None:
        """Business category: 事業内容の業種."""
        res = self.select_one('main h1:-soup-contains("基本情報") ~ div h2:-soup-contains("業種")')
        return re.sub(r"業種：", "", res.text)

    @webpage_property
    def business_description(self) -> str | None:
        """Business description: 事業内容の説明."""
        res = self.select_one('main h1:-soup-contains("基本情報") ~ div h2:-soup-contains("業種")+p')
        return re.sub(r"\s+", "", res.text)

    @webpage_property
    def products(self) -> list[str] | None:
        """Products: 取扱い商品."""
        res = self.select('main div:-soup-contains("取扱い商品") + div > p')
        return [re.sub(r"^・", "", r.text) for r in res]

    @webpage_property
    def segment_sales_composition(self) -> list[dict] | None:
        """Segment sales composition: セグメント売上構成."""
        rows = self.select('main div:-soup-contains("セグメント売上構成") + div table tr:has(td)')
        return [
            {
                "segment": r.find_all("td")[0].text,
                "sales": str2money(r.find_all("td")[1].text + "百万円"),
                "proportion": str2float(r.find_all("td")[2].text),
            }
            for r in rows
            if len(r.find_all("td")) == 3
            and r.find_all("td")[0].text != "損益計算書計上額"
            and r.find_all("td")[0].text != "調整額"
        ]

    @webpage_property
    def income_statement_amount(self) -> Money | None:
        """Income statement amount: 損益計算書計上額."""
        amount = self.select_one('main td:-soup-contains("損益計算書計上額") + td')
        return str2money(amount.text + "百万円")

    @webpage_property
    def income_statement_adjustment(self) -> Money | None:
        """Income statement adjustment: 調整額."""
        amount = self.select_one('main td:-soup-contains("調整額") + td')
        return str2money(amount.text + "百万円")

    @webpage_property
    def current_term_company_performance_forecast(self) -> str | None:
        """Current term company performance forecast: 業績予想 会社予想 今期見通し."""
        res = self.select_one(
            'main div:-soup-contains("業績予想") + div h2:-soup-contains("会社予想") + dl > dt:-soup-contains("今期見通し") + dd'
        )
        return re.sub(r"\s+", "", res.text)

    @webpage_property
    def analyst_company_performance_forecast_comparison(self) -> str | None:
        """Analyst forecast company forecast comparison: 業績予想 アナリスト予想 会社予想との比較."""
        res = self.select_one(
            'main div:-soup-contains("業績予想") + div h2:-soup-contains("アナリスト予想") + dl '
            '> dt:-soup-contains("会社予想との比較") + dd'
        )
        res = re.sub(r"\s+", "", res.text)
        return res if res != "--" else None

    @webpage_property
    def price_level_to_target(self) -> str | None:
        """Current price to target price: 目標株価に対する現在の価格が割高か割安か."""
        res = self.select_one(
            'main div:-soup-contains("目標株価") + div h2:-soup-contains("アナリスト評価") + div dt:-soup-contains("目標株価") + dd'
        )
        res = re.sub(r"\s+", "", res.text)
        return res if res != "--" else None

    @webpage_property
    def price_level_to_pbr_based_theoretical_price(self) -> str | None:
        """Price level to PBR based theoretical price: PBR基準の理論株価に対する現在の価格が割高か割安か."""
        res = self.select_one(
            'main div:-soup-contains("目標株価") + div h2:-soup-contains("理論株価") + div dt:-soup-contains("PBR基準") + dd'
        )
        if res is None:
            return None
        res = re.sub(r"\s+", "", res.text)
        return res if res != "--" else None

    @webpage_property
    def price_level_to_per_based_theoretical_price(self) -> str | None:
        """Price level to PER based theoretical price: PER基準の理論株価に対する現在の価格が割高か割安か."""
        res = self.select_one(
            'main div:-soup-contains("目標株価") + div h2:-soup-contains("理論株価") + div dt:-soup-contains("PER基準") + dd'
        )
        if res is None:
            return None
        res = re.sub(r"\s+", "", res.text)
        return res if res != "--" else None

    @webpage_property
    def risk_on_relative_index(self) -> str | None:
        """Risk on relative index: リスクオン相対指数."""
        res = self.select_one(
            'main div:-soup-contains("トレンドシグナル") + div h2:-soup-contains("リスクオン相対指数") + '
            'dl > dt:-soup-contains("水準") + dd'
        )
        if res is None:
            return None
        res = re.sub(r"\s+", "", res.text)
        return res if res != "--" else None

    @webpage_property
    def news_links(self) -> list[str]:
        """News links: ニュースのリンクのリスト."""
        res = self.select('main h1:-soup-contains("ニュース") + ul  a')
        if res is None:
            return []
        res = [r.get("href") for r in res]
        return [urllib.parse.urljoin(self.website.url, r) for r in res if isinstance(r, str)]



"""Scraper for https://kabuyoho.jp/sp/reportDps"""
from __future__ import annotations

import logging
import urllib.parse

from ..base import Website, webpage_property
from ..util import str2float
from .kabuyoho_webpage import KabuyohoWebpage

logger = logging.getLogger(__name__)


class ReportDps(KabuyohoWebpage):
    """Report target page object."""

    def __init__(self, website: Website, security_code: str | int) -> None:
        self.website = website
        self.security_code = str(security_code)
        self.url = urllib.parse.urljoin(self.website.url, f"sp/reportDps?bcode={self.security_code}")
        super().__init__()

    @webpage_property
    def actual_dividend_yield(self) -> float | None:
        """Actual dividend yield(実績配当利回り)."""
        amount = self.select_one('th:-soup-contains("実績配当利回り") + td')
        return str2float(amount.text)

    @webpage_property
    def expected_dividend_yield(self) -> float | None:
        """Expected dividend yield(予想配当利回り)."""
        amount = self.select_one('th:-soup-contains("予想配当利回り") + td')
        return str2float(amount.text)

    @webpage_property
    def dividend_payout_ratio(self) -> float | None:
        """Expected dividend yield(予想配当利回り)."""
        amount = self.select_one('h2:-soup-contains("前期配当性向") + div td')
        return str2float(amount.text)



"""Base classes."""
from __future__ import annotations

from .decorators import webpage_property
from .webpage import Webpage
from .website import Website

__all__ = ["Website", "Webpage", "webpage_property"]



"""Base class for website"""
from __future__ import annotations

from abc import ABC, abstractmethod


class Website(ABC):
    """Base class for website"""

    @abstractmethod
    def __init__(self):
        self.url: str



"""util functions"""
from __future__ import annotations

import re

from money import Money

__all__ = ["str2money", "str2float"]

jpy_unit = str.maketrans({"百": "0" * 2, "千": "0" * 3, "万": "0" * 4, "億": "0" * 8})


def str2money(price: str) -> Money | None:
    """Convert str to JPY Money object"""
    if not re.search(r"\d", price):
        return None
    amount = price.translate(jpy_unit)
    amount = re.sub(r"[^\d.-]", "", amount)
    if amount == "":
        return None
    return Money(amount, "JPY")


def str2float(value: str) -> float | None:
    """Convert str to float"""
    if not re.search(r"\d", value):
        return None
    amount = re.sub(r"[^\d.-]", "", value)
    if amount == "":
        return None
    return float(amount)


def str2int(value: str) -> int | None:
    """Convert str to int"""
    if not re.search(r"\d", value):
        return None
    amount = re.sub(r"[^\d.-]", "", value)
    if amount == "":
        return None
    return int(amount)
